
/*******************************************************************************************\
* Copyright 2002-2022 (C) Digital Multi-Media Design (DMMD), LLC (http://www.dmmd.net)      *
* This file is part of DMMD's Software Library.                                             *
* The software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; *
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. *
* DMMD can customize or expand this code for your own application.                          *
* Please contact us at contact@dmmd.net or via phone, at 7 0 3 - 4 3 9 - 0 0 6 7 (USA)      *
* or visit us at our website (http://dmmd.net).                                             *
\*******************************************************************************************/

var loadedImages = [];

jQuery(document).ready(function($)
{
    var loadedLargeImages = false;

    function arrayRotate(arr, count)
    {
        count -= arr.length * Math.floor(count / arr.length);
        arr.push.apply(arr, arr.splice(0, count));
    }

    function hotspotsRotate(count)
    {
        var firstNewIndex = imagePaths.length-count;

        for (var row=0; row<rowsCols.length; row++)
        {
            var hotspots = rowsCols[row];

            for (var h=0; h<hotspots.length; h++)
            {
                var hotspot = hotspots[h];

                hotspot.hotspotColumn = hotspot.hotspotColumn+firstNewIndex;
            }
        }
    }

    function loadImages(videoPath)
    {
        var count = -1;

        var video = document.createElement("video");
        var loadingBar = new ldBar(document.getElementById("progressElement"));

        video.addEventListener('loadeddata', function()
        {
            video.currentTime = 0;
        },
        false);

        video.addEventListener("seeked", function()
        {
            //console.log(count);
            //console.log(videoFrames);

            if (count < videoFrames)
            {
                count++;

                if (count > 0)
                {
                    generateFrame(count);

                    var progressValue = (100*count)/videoFrames;
                    loadingBar.set(progressValue, true);
                }

                var frameTime = video.duration/videoFrames;

                if (count == 0)
                {
                    video.currentTime += frameTime/2;
                }
                else
                {
                    video.currentTime += frameTime;
                }
            }
            else
            {
                document.getElementById("progressLoader").style.display = "none";
                mainFunction();
            }

        }, false);

        video.preload = "auto";
        video.src = videoPath;
        video.autoplay = true;
        video.muted = true;
        video.setAttribute("playsinline", "");

        function generateFrame(imgCount)
        {
            var c = document.createElement("canvas");
            var ctx = c.getContext("2d");

            c.width = normalWidth;
            c.height = normalHeight;
            ctx.drawImage(video, 0, 0, normalWidth, normalHeight);

            convertCanvasToImage(c, saveImage);
        }

        function convertCanvasToImage(canvas, callback)
        {
            var image = new Image();
            image.crossOrigin = "Anonymous";

            image.onload = function()
            {
                callback(image);
            };

            image.id = "videoframe-" + count;
            image.setAttribute("data-imgnumber", count);
            image.src = canvas.toDataURL("image/jpeg");
        }

        function saveImage(img)
        {
            //console.log("> "+img);
            loadedImages.push(img);
        }
    }



    function load360Export(){
    if (outputType === "html5")
    {
        //window.addEventListener("load", function()
        //{
            var loadingBar = new ldBar(document.getElementById("progressElement"));

            var loader = new PxLoader();

            var total = imagePaths.length;
            var count = 0;

            for (var i=0; i<total; i++)
            {
                var src = imagePaths[i];

                var pxImage = new PxLoaderImage(src);
                pxImage.imageNumber = i;
                loader.add(pxImage);
                loadedImages[i] = pxImage;
            }

            loader.addProgressListener(function(e)
            {
                var progressValue = (100*e.completedCount)/e.totalCount;
                loadingBar.set(progressValue, true);

                if (e.completedCount === e.totalCount)
                {
                    document.getElementById("progressLoader").style.display = "none";
                    mainFunction();
                }
            });

            loader.start();
        //});
    }
    else if (outputType === "interactivemp4")
    {
        //window.addEventListener("load", function()
        //{
            if (dynamicSize)
            {
                var normalVideoElement = document.createElement("video");

                normalVideoElement.addEventListener('loadeddata', function()
                {
                    normalWidth = normalVideoElement.videoWidth;
                    normalHeight = normalVideoElement.videoHeight;

                    largeWidth = normalVideoElement.videoWidth;
                    largeHeight = normalVideoElement.videoHeight;

                    //

                    if (deepZoom === 1)
                    {
                        var largeVideoElement = document.createElement("video");

                        largeVideoElement.addEventListener('loadeddata', function()
                        {
                            largeWidth = largeVideoElement.videoWidth;
                            largeHeight = largeVideoElement.videoHeight;

                            loadImages("img/video.mp4");
                        },
                        false);

                        largeVideoElement.preload = "auto";
                        largeVideoElement.autoplay = false;
                        largeVideoElement.muted = true;
                        largeVideoElement.setAttribute("playsinline", "");
                        largeVideoElement.src = "imglarge/video.mp4";
                    }
                    else
                    {
                        loadImages("img/video.mp4");
                    }
                },
                false);

                normalVideoElement.preload = "auto";
                normalVideoElement.autoplay = false;
                normalVideoElement.muted = true;
                normalVideoElement.setAttribute("playsinline", "");
                normalVideoElement.src = "img/video.mp4";
            }
            else
            {
                loadImages("img/video.mp4");
            }
        //});
    }
    }

    if (fff)
    {
        var topContainerE = document.getElementById("topContainer");

        //console.log(document.getElementById("topContainer"));

        var fimage = document.createElement("div");
        fimage.id = "fff-image";
//fimage.src = imagePaths[0];
fimage.style.backgroundImage = "url('" + imagePaths[0] + "')";
fimage.style.position = "relative";
fimage.style.backgroundRepeat = "no-repeat";
fimage.style.backgroundPosition = "center center";
fimage.style.backgroundSize = "contain";
fimage.style.zIndex = "300";
fimage.style.width = "100%";
fimage.style.height = "100%";

$(fimage).on("mouseup touchend", function (e)
{
fimage.remove();
load360Export();
});

topContainerE.appendChild(fimage);





    }
    else
    {
         window.addEventListener("load", function()
        {
        load360Export();
        });
    }


});

function mainFunction()
{
    var isTopWindow = false;
    var fullscreenControl = $("#fullscreen");
    var hotspotContainer = $("#hotspotContainer");
    var hotspotContainerElement = document.getElementById("hotspotContainer");
    var hotspotCanvas = $("#hotspotcanvas");
    var canvasContext = hotspotCanvas[0].getContext('2d');
    var hotspotCanvasElement = document.getElementById("hotspotcanvas");
    var playPauseControl = $("#play");
    var prevControl = $("#prev");
    var nextControl = $("#next");
    var downControl = $("#down");
    var upControl = $("#up");
    var zoomIn = $("#zoomin");
    var zoomCheck = $("#zoomcheck");
    var zoomPanel = $("#inverted-contain");
    var zoomImage = $("#zoomimg");
    var topContainer = $("#topContainer");
    var topContainerElement = document.getElementById("topContainer");
    var imageContainer = $("#imageContainer");
    var controllerContainer = $("#controllerContainer");
    var images = imageContainer.find("img");
    var imagesCount = loadedImages.length;
    var multiRowArray = new Array(numberOfRows);
    var isClicked = false;
    var currentPositionX;
    var currentPositionY;
    var currentImage = 0;
    var lastImageChanged = false;

    if (firstImageIndex > 0)
    {
        lastImageChanged = true;
        currentImage = firstImageIndex;
    }

    var previousImage = 0;

    var animation;

    var canvas;
    var radius = 15;

    // default initial zoom parameters
    var valueZoom = maxZoom;
    var minZoom = 1.0;
    valueZoom = minZoom;
    var stepZoom = 0.1;
    stepZoom *= zoomSensitivity;

    var currentZoomStep = 0;
    var zoomIncrement = true;

    var wasPlaying = false;

    var isHotspotImage = false;

    var isZooming = false;
    var isZoomClicked = false;
    var currentZoomXPosition = 0;
    var currentZoomYPosition = 0;
    var zoomBackgroundPosition = "center";

    var zoomDragPosX = 0;
    var zoomDragPosY = 0;

    var isBouncingFinished = false;
    var bounceRotationCount = 0;

    //

    var isAtTop = false;
    var isAtRight = false;
    var isAtBottom = false;
    var isAtLeft = false;

    var zoomedDraggableWidth = 0;
    var zoomedDraggableHeight = 0;

    var mouseDownXPosition = 0;
    var mouseDownYPosition = 0;

    var isFirstDrag = true;

    var hotspotImageWidth = 24;
    var hotspotImageHeight = 24;

    var touchZoomDistanceStart;
    var touchZoomDistanceEnd;

    var zoomTouchDistanceOld = 0;
    var stepPinchZoom = 0.1;

    //var currentRow = 0;
    var currentRow = firstRow-1;

    //

    var panzoomChanged = false;
    var panzoomTransform;

    var hotspotContainerPositionX;
    var hotspotContainerPositionY;

    //

    var newTempArr = [];
    var imagesTempArr = loadedImages.slice();

    while (imagesTempArr.length)
    {
        newTempArr.push(imagesTempArr.splice(0, imagesCount/numberOfRows));
    }

    for (var i=0; i<newTempArr[currentRow].length; i++)
    {
        var cImgNumber = -1;

        if (outputType === "html5")
        {
            cImgNumber = parseInt(newTempArr[currentRow][i].imageNumber);
        }
        else if (outputType === "interactivemp4")
        {
            cImgNumber = parseInt(newTempArr[currentRow][i].getAttribute("data-imgnumber"));
        }

        if (currentImage == cImgNumber)
        {
            currentImage = i;
        }
    }

    //

    var $panzoom;

    var zoomDivClose;
    var zoomDivCloseContainer;
    var zoomDivCloseDescription;

    //

    zoomImage.attr("src", "#");
    zoomImage.attr("display", "none");

    //

    hotspotContainer.css("cursor", "-webkit-"+cursorHoverName);
    hotspotContainer.css("cursor", cursorHoverName);

    //

    var friction = 0.85;

    var mouse = { x: 0, y: 0 };
    var position = { x: 0, y: 0 };
    var previous = { x: position.x, y: position.y };
    var velocity = { x: 0, y: 0 };

    //

    var overHotspot = false;

    //

    var opacity = 1;
    canvasContext.globalAlpha = 1;

    var fps = 30;
    var now;
    var then = Date.now();
    var interval = 1000/fps;
    var delta;

    //

    var logoImageSrc = $('#logocontainer').find('img:first').attr('src');

    //

    var tooltipDiv = $('#tooltip');

    //

    var newFrame = false;
    
    //
    
    var exitFromImageHotspot = false;

    //

    if (!isTopWindow)
    {
        var csIframe = getComputedStyle($("body")[0]);
        var widthIframe = parseInt(csIframe.getPropertyValue('width'), 10);
        var heightIframe = parseInt(csIframe.getPropertyValue('height'), 10);
        
        var aspectRatioWidth = (normalWidth*heightIframe)/normalHeight;
            
        normalWidth = aspectRatioWidth;
        normalHeight = heightIframe;
        hotspotsWidth = aspectRatioWidth;
        hotspotsHeight = heightIframe;
    }

    //

var cs = getComputedStyle($("#topContainer")[0]);
var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

    //Set up the dialog box
    $("#myDialog").dialog({
                              autoOpen: false,
                              modal: true,
                              draggable: false,
                              title: "",
                              maxWidth: widthCanvasTop,
                              maxHeight: heightCanvasTop,
                              width: widthCanvasTop,
                              height: heightCanvasTop,
                              appendTo: "#topContainer",
                              show: {
                                  effect: "blind",
                                  duration: 500
                              },
                              hide: {
                                  effect: "blind",
                                  duration: 500
                              }
                          });

$(".ui-dialog").css('padding', '0px');

    //

    function isValidImageUrl(url)
    {
        if (typeof url == 'undefined')
        {
            return false;
        }

        var image = new Image();
        image.src = url;

        if (image.width === 0)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    if (!isValidImageUrl(logoImageSrc))
    {
        $('#logocontainer').css({"visibility":"hidden"});
    }

    //

    if (self == top)
    {
        isTopWindow = true;

        if (deepZoom === 1)
        {
            $('img').prop('src', function ()
            {
                return this.src.replace("img/", "imglarge/");
            });
        }
    }
    else
    {
        isTopWindow = false;
    }

    if (!isTopWindow)
    {
        hotspotCanvas[0].width = normalWidth;
        hotspotCanvas[0].height = normalHeight;
    }
    else
    {
        hotspotCanvas[0].width = largeWidth;
        hotspotCanvas[0].height = largeHeight;
    }

    var cs = getComputedStyle(hotspotCanvas[0]);
    var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
    var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

    hotspotContainerElement.style.width = widthCanvasTop + "px";
    hotspotContainerElement.style.height = heightCanvasTop + "px";

    hotspotCanvas = $("#hotspotcanvas");
    canvasContext = hotspotCanvas[0].getContext('2d');

    var multiRowCount = 0;
    var imageRowCount = 0;

    for (var i=0; i<numberOfRows; i++)
    {
        multiRowArray[i] = new Array(imagesCount/numberOfRows);
        imageRowCount = 0;

        for (var j=multiRowCount; j<(multiRowCount+(imagesCount/numberOfRows)); j++)
        {
            if (outputType === "html5")
            {
                multiRowArray[i][imageRowCount] = loadedImages[j].img;
            }
            else if (outputType === "interactivemp4")
            {
                multiRowArray[i][imageRowCount] = loadedImages[j];
            }

            imageRowCount++;
        }

        multiRowCount += imagesCount/numberOfRows;
    }

    if (!isBlendingEnabled)
        {
            var img = multiRowArray[currentRow][currentImage];

            if (!isTopWindow)
            {
                canvasContext.drawImage(img, 0, 0, normalWidth, normalHeight);
            }
            else
            {
                canvasContext.drawImage(img, 0, 0, largeWidth, largeHeight);
            }

            displayHotspot(false);

        }
        else
        {
            fade1();
        }

    if (!isTopWindow)
    {
        hotspotImageWidth = (hotspotImageWidth*normalWidth)/400;
        hotspotImageHeight = hotspotImageWidth;
    }
    else
    {
        hotspotImageWidth = (hotspotImageWidth*largeWidth)/400;
        hotspotImageHeight = hotspotImageWidth;
    }

    var isMobileBrowser = function()
    {
        var mobileBrowser;
        var userAgent = navigator.userAgent;

        // regex literal, for other user agents, append their name in lower case
        var pattern = new RegExp('android|iphone|ipad|ipod|blackberry|iemobile|webos|opera mini');

        if (pattern.test(userAgent.toLowerCase()))
        {
            mobileBrowser = true;
        }
        else
        {
            mobileBrowser = false;
        }

        return mobileBrowser;
    };

    if (isMobileBrowser() && isTopWindow)
    {
        if (window.innerWidth < window.innerHeight)
        {
            hotspotCanvas.css({"width": "100%"});
            hotspotCanvas.css({"height": "auto"});

            hotspotCanvas.css({"top": "50%"});
            hotspotCanvas.css({"transform": "translateY(-50%)"});

            //

            hotspotContainer.css({"top": "50%"});
            hotspotContainer.css({"transform": "translateY(-50%)"});
        }
    }

    function hexToRgb(hex)
    {
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);

        return result ? {
                            r: parseInt(result[1], 16),
                            g: parseInt(result[2], 16),
                            b: parseInt(result[3], 16)
                        } : null;
    }

    function rgbToMatrix(rgb)
    {
        var red = rgb.r/255;
        var green = rgb.g/255;
        var blue = rgb.b/255;

        var matrix = '0 0 0 0 ' + red + '  0 0 0 0 ' + green + '  0 0 0 0 ' + blue + '  0 0 0 1 0';
        return matrix;
    }

    function colorizeElement(elementId, hexColor)
    {
        var rgb = hexToRgb(hexColor);
        var matrix = rgbToMatrix(rgb);

        var filter = document.querySelector('#colorFilter feColorMatrix');
        filter.setAttribute('values', matrix);

        document.getElementById(elementId).style.WebkitFilter = "url(#colorFilter)";
        document.getElementById(elementId).style.filter = "url(#colorFilter)";
    }

    function onOrientationChange()
    {
        if (isMobileBrowser() && !isTopWindow && isInFullscreen())
        {
            if (window.innerWidth > window.innerHeight)
            {
                // landscape

                hotspotCanvas.css({"width": "auto"});
                hotspotCanvas.css({"height": "100%"});

                hotspotCanvas.css({"left": "50%"});
                hotspotCanvas.css({"transform": "translateX(-50%)"});

                hotspotCanvas.css({"top": ""});
                hotspotCanvas.css({"margin": "0px"});

                //

                //hotspotContainer.css({"left": "50%"});
                //hotspotContainer.css({"transform": "translateX(-50%)"});
            }
            else
            {
                // portrait

                hotspotCanvas.css({"width": "100%"});
                hotspotCanvas.css({"height": "auto"});

                hotspotCanvas.css({"top": "50%"});
                hotspotCanvas.css({"transform": "translateY(-50%)"});

                hotspotCanvas.css({"left": ""});

                //

                hotspotContainer.css({"top": "50%"});
                hotspotContainer.css({"transform": "translateY(-50%)"});
            }
        }

        if (isMobileBrowser() && isTopWindow)
        {
            if (window.innerWidth > window.innerHeight)
            {
                // landscape

                hotspotCanvas.css({"width": "auto"});
                hotspotCanvas.css({"height": "100%"});

                hotspotCanvas.css({"left": "50%"});
                hotspotCanvas.css({"transform": "translateX(-50%)"});

                hotspotCanvas.css({"top": ""});
                hotspotCanvas.css({"margin": "0px"});
                
                //

                hotspotContainer.css({"top": "50%"});
                hotspotContainer.css({"transform": "translateY(-50%)"});
            }
            else
            {
                // portrait

                hotspotCanvas.css({"width": "100%"});
                hotspotCanvas.css({"height": "auto"});

                hotspotCanvas.css({"top": "50%"});
                hotspotCanvas.css({"transform": "translateY(-50%)"});

                hotspotCanvas.css({"left": ""});
                
                //

                hotspotContainer.css({"top": "50%"});
                hotspotContainer.css({"transform": "translateY(-50%)"});
            }
        }

        setTimeout(function()
        {
            var cs = getComputedStyle(hotspotCanvas[0]);
            var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
            var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

            //console.log(widthCanvasTop);

            hotspotContainer.css({"width": widthCanvasTop});
            hotspotContainer.css({"height": heightCanvasTop});

            reloadHotspotPositions(false);

        }, 500);
    }

    function leftPad(number, targetLength)
    {
        var output = number + '';

        while (output.length < targetLength)
        {
            output = '0' + output;
        }

        return output;
    }

    if (isMobileBrowser())
    {
        onOrientationChange();
    }

    $(window).on("orientationchange", function()
    {
        if (isMobileBrowser())
        {
            setTimeout(function()
            {
                onOrientationChange();
            }, 500);
        }
    });

    //

    hotspotContainer.on("mousedown touchstart", function (e)
    {
        if (!scrollEvent)
        {
            e.preventDefault();
        }

        if ($("#overlay").css("display") === "block")
        {
            $("#overlay").css("display", "none");
        }

        hotspotContainer.css("cursor", "-webkit-"+cursorActiveName);
        hotspotContainer.css("cursor", cursorActiveName);

        hideHotspotTitleText();

        if (isPlaying)
        {
            wasPlaying = true;
            doPause();
        }
        else
        {
            wasPlaying = false;
        }

        if (e.type === "touchstart")
        {
            var event = e || window.event;
            var touches = event.touches || event.originalEvent.touches;

            if (touches.length === 1)
            {
                currentPositionX = touches[0].pageX;
                currentPositionY = touches[0].pageY;
            }
            else if (touches.length === 2)
            {
                var dx = touches[0].pageX - touches[1].pageX;
                var dy = touches[0].pageY - touches[1].pageY;

                touchZoomDistanceEnd = touchZoomDistanceStart = Math.sqrt(dx * dx + dy * dy);
            }
        }
        else
        {
            currentPositionX = e.pageX;
            currentPositionY = e.pageY;
        }

        hotspotContainerPositionX = currentPositionX;
        hotspotContainerPositionY = currentPositionY;

        var offset = $(this).offset();

        var posRelativeX = e.pageX - offset.left;
        var posRelativeY = e.pageY - offset.top;

        if (e.type === "touchstart")
        {
            posRelativeX = touches[0].pageX - offset.left;
            posRelativeY = touches[0].pageY - offset.top;
        }

        if (isTopWindow)
        {
            var cs = getComputedStyle(hotspotCanvas[0]);
            var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
            var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

            posRelativeX = (posRelativeX * widthCanvasTop) / hotspotCanvas[0].offsetWidth;
            posRelativeY = (posRelativeY * heightCanvasTop) / hotspotCanvas[0].offsetHeight;
        }

        isClicked = true;
        return scrollEvent;
    });

    hotspotContainer.on("mouseup touchend", function (e)
    {
        hotspotContainer.css("cursor", "-webkit-"+cursorHoverName);
        hotspotContainer.css("cursor", cursorHoverName);

        if (isClicked && !isPlaying)
        {
            isClicked = false;

            if (isBlendingEnabled)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
                fade1();
            }

            if (isSingleRotation)
            {
                doPause();
            }
            else
            {
                if (wasPlaying)
                {
                    doPlay();
                }
                else
                {
                    doPause();
                }
            }
        }

        if (!isRotationAfterMouseControl)
        {
            doPause();
        }

        if (e.type === "touchend")
        {
            var event = e || window.event;
            var touches = event.touches || event.originalEvent.touches;

            if (touches.length === 1)
            {
                currentPositionX = touches[0].pageX;
                currentPositionY = touches[0].pageY;
            }
            else if (touches.length === 2)
            {
                var dx = touches[0].pageX - touches[1].pageX;
                var dy = touches[0].pageY - touches[1].pageY;

                touchZoomDistanceEnd = touchZoomDistanceStart = Math.sqrt(dx * dx + dy * dy);
            }
        }
        else
        {
            currentPositionX = e.pageX;
            currentPositionY = e.pageY;
        }

        if ((currentPositionX == hotspotContainerPositionX) && (currentPositionY == hotspotContainerPositionY))
        {
            if (zoomedRotation && isZooming)
            {
                if (hotspotContainer.css("visibility") === "hidden")
                {
                    hotspotContainer.css("visibility", "visible");
                }
                else
                {
                    hotspotContainer.css("visibility", "hidden");
                }
            }
        }

        return scrollEvent;
    });

    hotspotContainer.mouseout(function()
    {
        hotspotContainer.css("cursor", "-webkit-"+cursorHoverName);
        hotspotContainer.css("cursor", cursorHoverName);

        if (isClicked && !isPlaying)
        {
            isClicked = false;

            if (isBlendingEnabled)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
                fade1();
            }

            if (isSingleRotation)
            {
                doPause();
            }
            else
            {
                if (wasPlaying)
                {
                    doPlay();
                }
                else
                {
                    doPause();
                }
            }
        }
    });

    hotspotContainer.on("mousemove touchmove", function (e)
    {
        var offset = $(this).offset();

        var posRelativeX = e.pageX - offset.left;
        var posRelativeY = e.pageY - offset.top;

        if (e.type === "touchmove")
        {
            var eventTouch = e || window.event;
            var touchesEv = eventTouch.touches || eventTouch.originalEvent.touches;

            posRelativeX = touchesEv[0].pageX - offset.left;
            posRelativeY = touchesEv[0].pageY - offset.top;
        }

        mouse.x = posRelativeX;
        mouse.y = posRelativeY;

        //

        if (isClicked)
        {
            var xPosition;
            var yPosition;

            hotspotContainer.css("cursor", "-webkit-"+cursorActiveName);
            hotspotContainer.css("cursor", cursorActiveName);

            if (e.type === "touchmove")
            {
                if (!scrollEvent)
                {
                    e.preventDefault();
                }

                var event = e || window.event;
                var touches = event.touches || event.originalEvent.touches;

                if (touches.length === 1)
                {
                    xPosition = touches[0].pageX;
                    yPosition = touches[0].pageY;
                }
                else if (touches.length === 2)
                {
                    var dx = touches[0].pageX - touches[1].pageX;
                    var dy = touches[0].pageY - touches[1].pageY;

                    touchZoomDistanceEnd = Math.sqrt(dx * dx + dy * dy);

                    var factor = touchZoomDistanceStart / touchZoomDistanceEnd;
                    touchZoomDistanceStart = touchZoomDistanceEnd;

                    if (touchZoomDistanceEnd > zoomTouchDistanceOld)
                    {
                        valueZoom += stepPinchZoom;
                        valueZoom = (valueZoom>maxZoom) ? maxZoom : valueZoom;
                    }
                    else
                    {
                        valueZoom -= stepPinchZoom;
                        valueZoom = (valueZoom<minZoom) ? minZoom : valueZoom;
                    }

                    if (valueZoom <= maxZoom && valueZoom >= minZoom)
                    {
                        //document.getElementById("log").innerHTML = valueZoom;
                        //doZoom(false, "", "", false);

                        if (outputType === "html5")
                        {
                            doZoom(false, "", "", false, null);
                        }
                        else if (outputType === "interactivemp4")
                        {
                            loadLargeImage(currentImage, doZoomCallback);
                        }
                    }

                    if (valueZoom < 1.1)
                    {
                        exitZoom(false);
                    }

                    zoomTouchDistanceOld = touchZoomDistanceEnd;
                }
            }
            else
            {
                xPosition = e.pageX;
                yPosition = e.pageY;
            }

            if (!isInertiaEnabled)
            {
                if (Math.abs(currentPositionX - xPosition) >= sensitivity)
                {
                    if (currentPositionX - xPosition >= sensitivity)
                    {
                        if (isPointerDragNormal)
                        {
                            displayPreviousFrame();
                        }
                        else
                        {
                            displayNextFrame();
                        }
                    }
                    else
                    {
                        if (isPointerDragNormal)
                        {
                            displayNextFrame();
                        }
                        else
                        {
                            displayPreviousFrame();
                        }
                    }

                    currentPositionX = xPosition;
                }
            }
        }

        if ( (numberOfRows > 1) && (Math.abs(currentPositionY - yPosition) >= (sensitivityVertical * 10)))
        {
            if (currentPositionY - yPosition >= (sensitivityVertical * 10))
            {
                if (isPointerDragNormal)
                {
                    console.log("down");

                    currentRow--;

                    if (currentRow < 0)
                    {
                        currentRow = 0;
                    }

                    displayRowFrame();
                }
                else
                {
                    currentRow++;

                    if (currentRow > (numberOfRows-1))
                    {
                        currentRow = numberOfRows-1;
                    }

                    console.log("up");
                    displayRowFrame();
                }
            }
            else
            {
                if (isPointerDragNormal)
                {
                    currentRow++;

                    if (currentRow > (numberOfRows-1))
                    {
                        currentRow = numberOfRows-1;
                    }

                    //console.log("up");
                    displayRowFrame();
                }
                else
                {
                    currentRow--;

                    if (currentRow < 0)
                    {
                        currentRow = 0;
                    }

                    //console.log("down");
                    displayRowFrame();
                }
            }

            currentPositionY = yPosition;
        }

        return scrollEvent;
    });

    hotspotContainer.on("wheel", function(e)
    {
        if (mouseZoom)
        {
            var delta = e.delta || e.originalEvent.wheelDelta;
            var zoomOut = delta ? delta < 0 : e.originalEvent.deltaY > 0;

            if (zoomOut)
            {
                if (isPlaying)
                {
                    doPause();
                }

                if (!isZooming)
                {
                    zoomIn.removeClass("zoomin");
                    zoomIn.addClass("zoomout");

                    //valueZoom = maxZoom;
                    //doZoom(false, "", "", true);

                    if (outputType === "html5")
                    {
                        doZoomWheel(false, "", "", false, null);
                    }
                    else if (outputType === "interactivemp4")
                    {
                        loadLargeImage(currentImage, doZoomCallback);
                    }
                }
            }
        }
    });

    if (document.getElementById("play"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("play", colorizeColor);
        }

        document.getElementById("play").addEventListener('click', function(e)
        {
            if (isBlendingEnabled)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
                fade1();
            }

            if (!isPlaying)
            {
                doPlay();
            }
            else
            {
                doPause();
            }
        }, false);
    }

    if (document.getElementById("prev"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("prev", colorizeColor);
        }

        document.getElementById("prev").addEventListener('click', function(e)
        {
            displayPreviousFrame();

            if (isBlendingEnabled)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
                fade1();
            }

            if (isPlaying)
            {
                doPause();
            }

        }, false);
    }

    if (document.getElementById("next"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("next", colorizeColor);
        }

        document.getElementById("next").addEventListener('click', function(e)
        {
            displayNextFrame();

            if (isBlendingEnabled)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
                fade1();
            }

            if (isPlaying)
            {
                doPause();
            }

        }, false);
    }

    if (document.getElementById("down"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("down", colorizeColor);
        }

        document.getElementById("down").addEventListener('click', function(e)
        {
            if (isPointerDragNormal)
            {
                currentRow--;

                if (currentRow < 0)
                {
                    currentRow = 0;
                }

                displayRowFrame();
            }
            else
            {
                currentRow++;

                if (currentRow > (numberOfRows-1))
                {
                    currentRow = numberOfRows-1;
                }

                displayRowFrame();
            }

        }, false);
    }

    if (document.getElementById("up"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("up", colorizeColor);
        }

        document.getElementById("up").addEventListener('click', function(e)
        {
            if (isPointerDragNormal)
            {
                currentRow++;

                if (currentRow > (numberOfRows-1))
                {
                    currentRow = numberOfRows-1;
                }

                displayRowFrame();
            }
            else
            {
                currentRow--;

                if (currentRow < 0)
                {
                    currentRow = 0;
                }

                displayRowFrame();
            }

        }, false);
    }

    if (document.getElementById("zoomin"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("zoomin", colorizeColor);
        }

        document.getElementById("zoomin").addEventListener('click', function(e)
        {
            if (isBlendingEnabled)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
                fade1();
            }

            if (isPlaying)
            {
                doPause();
            }



if (currentZoomStep >= zoomSteps)
{
    zoomIncrement = false;
}

if (currentZoomStep < 1)
{
    zoomIncrement = true;
}

//

var zoomDiff = maxZoom - minZoom;
var zoomStep = zoomDiff / zoomSteps;

if (zoomIncrement)
{
    currentZoomStep++;
    valueZoom += zoomStep;
}
else
{
    currentZoomStep--;
    valueZoom -= zoomStep;
}

//

if (currentZoomStep >= zoomSteps)
{
    zoomIn.removeClass("zoomin");
    zoomIn.addClass("zoomout");
}

//console.log(valueZoom);

                if (outputType === "html5")
                {
                    doZoom(false, "", "", false, null);
                }
                else if (outputType === "interactivemp4")
                {
                    loadLargeImage(currentImage, doZoomCallback);
                }

if (exitFromImageHotspot)
{
exitFromImageHotspot = false;
currentZoomStep = 1;

//console.log(valueZoom);kaka

if (isMobileBrowser() && isTopWindow)
                    {
                        zoomImage.css("top", "50%");
                        
                        zoomImage.css("width", "auto");
                        zoomImage.css("height", "100%");
                    }

var cs = getComputedStyle(hotspotCanvas[0]);
var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

//hotspotContainer.css({"width": widthCanvasTop});
//hotspotContainer.css({"height": heightCanvasTop});
    
//reloadHotspotPositions(false);
console.log($panzoom[0].style.transform);

hotspotContainerElement.style.transform = $panzoom[0].style.transform;
hotspotContainerElement.style.transformOrigin = $panzoom[0].style.transformOrigin;
hotspotContainerElement.style.cursor = $panzoom[0].style.cursor;
hotspotContainerElement.style.transition = $panzoom[0].style.transition;
}

if (currentZoomStep < minZoom)
{
    zoomIn.removeClass("zoomout");
    zoomIn.addClass("zoomin");

    exitZoom(false);

    var cs = getComputedStyle(hotspotCanvas[0]);
    var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
    var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

    hotspotContainer.css({"width": widthCanvasTop});
    hotspotContainer.css({"height": heightCanvasTop});

    reloadHotspotPositions(false);
}



        }, false);
    }

    if (document.getElementById("fullscreen"))
    {
        if (isColorizeEnabled)
        {
            colorizeElement("fullscreen", colorizeColor);
        }

        document.getElementById("fullscreen").addEventListener('click', function(e)
        {
            if (screenfull.isEnabled)
            {
                var topContainerElem = topContainer[0];
                screenfull.toggle(topContainerElem);


                onFullscreenChangeEvent();
                //reloadHotspotPositions();
            }
        }, false);
    }

    document.addEventListener("fullscreenchange", function()
    {
        if (isMobileBrowser())
        {
            onFullscreenChangeEvent();
        }
    });

    document.addEventListener("mozfullscreenchange", function()
    {
        if (isMobileBrowser())
        {
            onFullscreenChangeEvent();
        }
    });

    document.addEventListener("webkitfullscreenchange", function()
    {
        if (isMobileBrowser())
        {
            onFullscreenChangeEvent();
        }
    });

    document.addEventListener("msfullscreenchange", function()
    {
        if (isMobileBrowser())
        {
            onFullscreenChangeEvent();
        }
    });

    function onFullscreenChangeEvent()
    {
        var windowWidth = window.innerWidth;
        var windowHeight = window.innerHeight;

        if (window.innerWidth > window.innerHeight)
        {
            // landscape

            hotspotCanvas.css({"width": "auto"});
            hotspotCanvas.css({"height": "100%"});
            hotspotCanvas.css({"max-width": "100%"});
            hotspotCanvas.css({"object-fit": "contain"});

            //hotspotCanvas.css({"left": "50%"});
            //hotspotCanvas.css({"transform": "translateX(-50%)"});

            //
            /*
            hotspotContainer.css({"width": "auto"});
            hotspotContainer.css({"height": "100%"});
            hotspotContainer.css({"max-width": "100%"});
            hotspotContainer.css({"object-fit": "contain"});

            hotspotContainer.css({"left": "50%"});
            hotspotContainer.css({"transform": "translateX(-50%)"});
            */
        }
        else if (window.innerWidth < window.innerHeight)
        {
            // portrait

            hotspotCanvas.css({"width": "100%"});
            hotspotCanvas.css({"height": "auto"});
            hotspotCanvas.css({"max-height": "100%"});
            hotspotCanvas.css({"object-fit": "contain"});

            //hotspotCanvas.css({"top": "50%"});
            //hotspotCanvas.css({"transform": "translateY(-50%)"});

            //
            /*
            hotspotContainer.css({"width": "100%"});
            hotspotContainer.css({"height": "auto"});
            hotspotContainer.css({"max-height": "100%"});
            hotspotContainer.css({"object-fit": "contain"});

            hotspotContainer.css({"top": "50%"});
            hotspotContainer.css({"transform": "translateY(-50%)"});
            */
        }

        setTimeout(function()
        {
            var cs = getComputedStyle(hotspotCanvas[0]);
            var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
            var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

            //console.log(widthCanvasTop);

            hotspotContainer.css({"width": widthCanvasTop});
            hotspotContainer.css({"height": heightCanvasTop});

            reloadHotspotPositions(false);

        }, 500);



        //

        if (isMobileBrowser())
        {
            setTimeout(function()
            {
                onOrientationChange();
            }, 500);
        }
    }

    function reloadHotspotPositions(zoomedRotation)
    {
        hotspotContainerElement.innerHTML = "";
        displayHotspot(zoomedRotation);
    }

    function isInFullscreen()
    {
        return screenfull.isFullscreen;
    }

    function doZoomCallback(callback)
    {
        callback();
    }

    function loadLargeImage(frameNumber, largeImgCallback)
    {
        frameNumber+=1;

        var video = document.createElement("video");

        video.addEventListener('loadeddata', function()
        {
            video.currentTime = 0;

            var frameTime = video.duration/videoFrames;

            video.currentTime = (frameTime*frameNumber)-(frameTime/2);

            video.addEventListener("seeked", function()
            {
                generateImageFrame(frameNumber);
            }, false);
        },
        false);

        video.preload = "auto";

        if (deepZoom === 1)
        {
            video.src = "imglarge/video.mp4";
        }
        else
        {
            video.src = "img/video.mp4";
        }

        video.autoplay = false;
        video.muted = true;
        video.setAttribute("playsinline", "");

        function generateImageFrame(imgCount)
        {
            var c = document.createElement("canvas");
            var ctx = c.getContext("2d");

            c.width = largeWidth;
            c.height = largeHeight;
            ctx.drawImage(video, 0, 0, largeWidth, largeHeight);

            convertCanvasToImg(c, saveImg);
        }

        function convertCanvasToImg(canvas, callback)
        {
            var image = new Image();
            image.crossOrigin = "Anonymous";

            image.onload = function()
            {
                callback(image);
            };

            image.id = "largeframe-" + frameNumber;
            image.src = canvas.toDataURL("image/jpeg");
            image.setAttribute("data-imgnumber", frameNumber);
        }

        function saveImg(img)
        {
            largeImgCallback(function()
            {
                doZoom(false, "", "", false, img);
            });

        }
    }

    function doZoom(hotspot, imageName, description, wheel, largeimg)
    {
        isZooming = true;

        hideHotspotTitleText();

        //zoomIn.removeClass("zoomin");
        //zoomIn.addClass("zoomout");

        doPause();

        //valueZoom = Math.round(valueZoom*10)/10;

        var multiRowFileName;
        var multiRowImg;

        if (outputType === "html5")
        {
            multiRowFileName = $(multiRowArray[currentRow][currentImage]).attr("src").split("/").pop();
        }
        else if (outputType === "interactivemp4")
        {
            multiRowImg = largeimg.src;
        }

        if ((deepZoom === 1) && !hotspot)
        {
            var imgDir = "imglarge";

            if (zoomedRotation)
            {
                imgDir = "img";

                var origImage = new Image();
                origImage.onload = function()
                {
                    zoomImage.attr("src", this.src);
                };

                origImage.src = "imglarge" + "/" + multiRowFileName;
            }


            if (isTopWindow)
            {
                //var largeImageSrc = images.eq(currentImage).attr("src");
                //var largeImageLastSeparator = largeImageSrc.lastIndexOf("/");
                //var largeImageFilename = largeImageSrc.substring(largeImageLastSeparator+1);

                //console.log(largeImageFilename);
                //draggableContainer.css("background-image", "url(imglarge/" + multiRowFileName + ")");
                //zoomImage.src =  "imglarge/" + multiRowFileName;

                if (outputType === "html5")
                {
                    zoomImage.attr("src", imgDir + "/" + multiRowFileName);
                }
                else if (outputType === "interactivemp4")
                {
                    zoomImage.attr("src", multiRowImg);
                }
            }
            else
            {
                //draggableContainer.css("background-image", "url(imglarge/" + multiRowFileName + ")");
                //zoomImage.src =  "imglarge/" + multiRowFileName;

                if (outputType === "html5")
                {
                    zoomImage.attr("src", imgDir + "/" + multiRowFileName);
                }
                else if (outputType === "interactivemp4")
                {
                    zoomImage.attr("src", multiRowImg);
                }
            }
        }
        else
        {
            //draggableContainer.css("background-image", "url(img/" + multiRowFileName + ")");
            //zoomImage.src =  "img/" + multiRowFileName;
            //zoomImage.attr("src", "img/" + multiRowFileName);

            if (outputType === "html5")
            {
                zoomImage.attr("src", "img/" + multiRowFileName);
            }
            else if (outputType === "interactivemp4")
            {
                zoomImage.attr("src", multiRowImg);
            }
        }

        if (hotspot)
        {
            zoomImage.attr("src", "hotspotimg/" + imageName);
        }

        //draggableContainer.css("background-repeat", "no-repeat");
        //draggableContainer.css("background-position", zoomBackgroundPosition);
        //draggableContainer.draggable();
        zoomImage.css("display", "block");
        zoomPanel.css("display", "block");
        imageContainer.css("display", "none");

        hotspotCanvas.css("visibility", "hidden");

        if (hotspot)
        {
            hotspotContainer.css("visibility", "hidden");
            playPauseControl.css("visibility", "hidden");
            prevControl.css("visibility", "hidden");
            nextControl.css("visibility", "hidden");

            //

            zoomIn.css("visibility", "hidden");
        }



        //
        //

        (function()
        {
            var $section = $('#inverted-contain');

            $panzoom = $section.find('.panzoom').panzoom({
                                                             startTransform: 'scale(' + 1.0 + ')',
                                                             //increment: 0.1,
                                                             //rangeStep: 0.1,
                                                             minScale: 1.0,
                                                             maxScale: maxZoom,
                                                             contain: 'invert'
                                                         }).panzoom();



            $panzoom.on("panzoomstart", function(e)
                    {
                        panzoomChanged = true;

                        panzoomTransform = $panzoom[0].style.transform;
                    });
            $panzoom.on("panzoomend", function(e)
                    {
                        if (panzoomChanged)
                        {
                            panzoomChanged = false;

                            if ($panzoom[0].style.transform == panzoomTransform)
                            {
                                if (zoomedRotation && isZooming)
                                {
                                    if (hotspotContainer.css("visibility") === "hidden")
                                    {
                                        hotspotContainer.css("visibility", "visible");
                                    }
                                    else
                                    {
                                        hotspotContainer.css("visibility", "hidden");
                                    }
                                }
                                
                                hotspotContainerElement.style.transform = $panzoom[0].style.transform;
                                hotspotContainerElement.style.transformOrigin = $panzoom[0].style.transformOrigin;
                                hotspotContainerElement.style.cursor = $panzoom[0].style.cursor;
                                hotspotContainerElement.style.transition = $panzoom[0].style.transition;
                            }
                        }
                    });

            if (hotspot)
            {
                $panzoom.panzoom('zoom', parseFloat(1.0),
                                 {
                                     animate: false,
                                     //focal: e
                                 });

                $('.zoomdivclose').remove();

                zoomDivClose = document.createElement('div');
                zoomDivClose.id = 'zoomdivclose';
                zoomDivClose.classList.add("zoomdivclose");

                var hotspotImageZoomButton = $(zoomDivClose);
                hotspotImageZoomButton.on("mouseup touchend", function (e)
                {
                    e.stopPropagation();
                    exitZoom(true);
                    topContainerElement.removeChild(zoomDivClose);
                });

                zoomDivCloseDescription = document.createElement('p');
                zoomDivCloseDescription.id = 'zoomdivclose-desc';
                zoomDivCloseDescription.innerHTML = description;

                zoomDivCloseContainer = document.createElement('div');
                zoomDivCloseContainer.id = 'zoomdivclose-container';

                zoomDivCloseContainer.appendChild(zoomDivCloseDescription);
                zoomDivClose.appendChild(zoomDivCloseContainer);
                topContainerElement.appendChild(zoomDivClose);

                if (hotspot)
                {
                    if (!isTopWindow)
                    {
                        if (hotspotsImageWidth > hotspotsImageHeight)
                        {
                            zoomImage.css({"width": "100%"});
                            zoomImage.css({"height": "auto"});

                            zoomImage.css({"top": "50%"});
                            zoomImage.css({"transform": "translate(-50%, -50%)"});
                        }
                        else
                        {
                            zoomImage.css({"width": "auto"});
                            zoomImage.css({"height": "100%"});
                        }
                    }
					else
                    {
                        if (hotspotsImageWidth > hotspotsImageHeight)
                        {
                            zoomImage.css({"width": "100%"});
                            zoomImage.css({"height": "auto"});

                            zoomImage.css({"top": "50%"});
                            zoomImage.css({"transform": "translate(-50%, -50%)"});
                        }
                        else
                        {
                            zoomImage.css({"width": "auto"});
                            zoomImage.css({"height": "100%"});
                        }
                    }
                }
            }
            else
            {
                if (wheel)
                {
                    valueZoom = 1.0;

                    /*$panzoom = $section.find('.panzoom').panzoom({
                                                                     startTransform: 'scale(' + 1.0 + ')',
                                                                     //increment: 0.1,
                                                                     //rangeStep: 0.1,
                                                                     minScale: 1.0,
                                                                     maxScale: maxZoom,
                                                                     contain: 'invert'
                                                                 }).panzoom();*/

                    $panzoom.panzoom('zoom', parseFloat(valueZoom),
                                     {
                                         animate: false,
                                         //focal: e
                                     });
                }
                else
                {
                    //valueZoom = maxZoom;

                    /*$panzoom = $section.find('.panzoom').panzoom({
                                                                     startTransform: 'scale(' + maxZoom + ')',
                                                                     //increment: 0.1,
                                                                     //rangeStep: 0.1,
                                                                     minScale: 1.0,
                                                                     maxScale: maxZoom,
                                                                     contain: 'invert'
                                                                 }).panzoom();*/

                    $panzoom.panzoom('zoom', parseFloat(valueZoom),
                                     {
                                         animate: false,
                                         //focal: e
                                     });
                }

                if (mouseZoom)
                {
                    $panzoom.parent().on('mousewheel.focal', function( e )
                    {
                        e.preventDefault();

                        var delta = e.delta || e.originalEvent.wheelDelta;
                        var zoomOut = delta ? delta < 0 : e.originalEvent.deltaY > 0;

                        if (zoomOut)
                        {
                            //scroll down
                            valueZoom += stepZoom;
                            valueZoom = (valueZoom>maxZoom) ? maxZoom : valueZoom;
                        }
                        else
                        {
                            //scroll up
                            valueZoom -= stepZoom;
                            valueZoom = (valueZoom<minZoom) ? minZoom : valueZoom;
                        }

                        if (valueZoom <= minZoom)
                        {
                            zoomIn.removeClass("zoomout");
                            zoomIn.addClass("zoomin");

                            //valueZoom = minZoom;
                            exitZoom(false);
                        }

                        if ((valueZoom >= minZoom) && (valueZoom < maxZoom))
                        {
                            $panzoom.panzoom('zoom', parseFloat(valueZoom),
                                             {
                                                 animate: false,
                                                 focal: e
                                             });
                        }

                    });
                }

                //if (zoomedRotation)
                //{
                    if (isMobileBrowser() && isTopWindow)
                    {
                        reloadHotspotPositions(true);
                    }
                
                    hotspotContainerElement.style.transform = $panzoom[0].style.transform;
                    hotspotContainerElement.style.transformOrigin = $panzoom[0].style.transformOrigin;
                    hotspotContainerElement.style.cursor = $panzoom[0].style.cursor;
                    hotspotContainerElement.style.transition = $panzoom[0].style.transition;
                    
                    if (isMobileBrowser() && isTopWindow)
                    {
                        hotspotContainer.css("top", "");
                        
                        hotspotContainer.css("width", "auto");
                        hotspotContainer.css("height", "100%");
                    }
                //}
            }

        })();
    }

    function doZoomWheel(hotspot, imageName, description, wheel, largeimg)
    {
        isZooming = true;

        hideHotspotTitleText();

        zoomIn.removeClass("zoomin");
        zoomIn.addClass("zoomout");

        doPause();

        valueZoom = Math.round(valueZoom*10)/10;

        var multiRowFileName;
        var multiRowImg;

        if (outputType === "html5")
        {
            multiRowFileName = $(multiRowArray[currentRow][currentImage]).attr("src").split("/").pop();
        }
        else if (outputType === "interactivemp4")
        {
            multiRowImg = largeimg.src;
        }

        if (deepZoom === 1)
        {
            if (isTopWindow)
            {
                //var largeImageSrc = images.eq(currentImage).attr("src");
                //var largeImageLastSeparator = largeImageSrc.lastIndexOf("/");
                //var largeImageFilename = largeImageSrc.substring(largeImageLastSeparator+1);

                //console.log(largeImageFilename);
                //draggableContainer.css("background-image", "url(imglarge/" + multiRowFileName + ")");
                //zoomImage.src =  "imglarge/" + multiRowFileName;

                if (outputType === "html5")
                {
                    zoomImage.attr("src", "imglarge/" + multiRowFileName);
                }
                else if (outputType === "interactivemp4")
                {
                    zoomImage.attr("src", multiRowImg);
                }
            }
            else
            {
                //draggableContainer.css("background-image", "url(imglarge/" + multiRowFileName + ")");
                //zoomImage.src =  "imglarge/" + multiRowFileName;

                if (outputType === "html5")
                {
                    zoomImage.attr("src", "imglarge/" + multiRowFileName);
                }
                else if (outputType === "interactivemp4")
                {
                    zoomImage.attr("src", multiRowImg);
                }
            }
        }
        else
        {
            //draggableContainer.css("background-image", "url(img/" + multiRowFileName + ")");
            //zoomImage.src =  "img/" + multiRowFileName;
            //zoomImage.attr("src", "img/" + multiRowFileName);

            if (outputType === "html5")
            {
                zoomImage.attr("src", "img/" + multiRowFileName);
            }
            else if (outputType === "interactivemp4")
            {
                zoomImage.attr("src", multiRowImg);
            }
        }

        if (hotspot)
        {
            zoomImage.attr("src", "hotspotimg/" + imageName);
        }

        //draggableContainer.css("background-repeat", "no-repeat");
        //draggableContainer.css("background-position", zoomBackgroundPosition);
        //draggableContainer.draggable();
        zoomImage.css("display", "block");
        zoomPanel.css("display", "block");
        imageContainer.css("display", "none");
        hotspotCanvas.css("visibility", "hidden");
        hotspotContainer.css("visibility", "hidden");
        playPauseControl.css("visibility", "hidden");
        prevControl.css("visibility", "hidden");
        nextControl.css("visibility", "hidden");

        if (hotspot)
        {
            zoomIn.css("visibility", "hidden");
        }

        //
        //

        (function()
        {
            var $section = $('#inverted-contain');

            $panzoom = $section.find('.panzoom').panzoom({
                                                             startTransform: 'scale(' + 1.0 + ')',
                                                             //increment: 0.1,
                                                             //rangeStep: 0.1,
                                                             minScale: 1.0,
                                                             maxScale: maxZoom,
                                                             contain: 'invert'
                                                         }).panzoom();

            if (hotspot)
            {
                $panzoom.panzoom('zoom', parseFloat(1.0),
                                 {
                                     animate: false,
                                     //focal: e
                                 });

                $('.zoomdivclose').remove();

                zoomDivClose = document.createElement('div');
                zoomDivClose.id = 'zoomdivclose';
                zoomDivClose.classList.add("zoomdivclose");

                var hotspotImageZoomButton = $(zoomDivClose);
                hotspotImageZoomButton.on("mouseup touchend", function (e)
                {
                    e.stopPropagation();
                    exitZoom(true);
                    topContainerElement.removeChild(zoomDivClose);
                });

                zoomDivCloseDescription = document.createElement('p');
                zoomDivCloseDescription.id = 'zoomdivclose-desc';
                zoomDivCloseDescription.innerHTML = description;

                zoomDivCloseContainer = document.createElement('div');
                zoomDivCloseContainer.id = 'zoomdivclose-container';

                zoomDivCloseContainer.appendChild(zoomDivCloseDescription);
                zoomDivClose.appendChild(zoomDivCloseContainer);
                topContainerElement.appendChild(zoomDivClose);

                if (hotspot)
                {
                    if (!isTopWindow)
                    {
                        if (hotspotsImageWidth > hotspotsImageHeight)
                        {
                            zoomImage.css({"width": "100%"});
                            zoomImage.css({"height": "auto"});

                            zoomImage.css({"top": "50%"});
                            zoomImage.css({"transform": "translate(-50%, -50%)"});
                        }
                        else
                        {
                            zoomImage.css({"width": "auto"});
                            zoomImage.css({"height": "100%"});
                        }
                    }
                }
            }
            else
            {
                if (wheel)
                {
                    valueZoom = 1.0;

                    /*$panzoom = $section.find('.panzoom').panzoom({
                                                                     startTransform: 'scale(' + 1.0 + ')',
                                                                     //increment: 0.1,
                                                                     //rangeStep: 0.1,
                                                                     minScale: 1.0,
                                                                     maxScale: maxZoom,
                                                                     contain: 'invert'
                                                                 }).panzoom();*/

                    $panzoom.panzoom('zoom', parseFloat(valueZoom),
                                     {
                                         animate: false,
                                         //focal: e
                                     });
                }
                else
                {
                    valueZoom = maxZoom;

                    /*$panzoom = $section.find('.panzoom').panzoom({
                                                                     startTransform: 'scale(' + maxZoom + ')',
                                                                     //increment: 0.1,
                                                                     //rangeStep: 0.1,
                                                                     minScale: 1.0,
                                                                     maxScale: maxZoom,
                                                                     contain: 'invert'
                                                                 }).panzoom();*/

                    $panzoom.panzoom('zoom', parseFloat(maxZoom),
                                     {
                                         animate: false,
                                         //focal: e
                                     });
                }

                if (mouseZoom)
                {
                    $panzoom.parent().on('mousewheel.focal', function( e )
                    {
                        e.preventDefault();

                        var delta = e.delta || e.originalEvent.wheelDelta;
                        var zoomOut = delta ? delta < 0 : e.originalEvent.deltaY > 0;

                        if (zoomOut)
                        {
                            //scroll down
                            valueZoom += stepZoom;
                            valueZoom = (valueZoom>maxZoom) ? maxZoom : valueZoom;
                        }
                        else
                        {
                            //scroll up
                            valueZoom -= stepZoom;
                            valueZoom = (valueZoom<minZoom) ? minZoom : valueZoom;
                        }

                        if (valueZoom <= minZoom)
                        {
                            zoomIn.removeClass("zoomout");
                            zoomIn.addClass("zoomin");

                            //valueZoom = minZoom;
                            exitZoom(false);
                        }

                        if ((valueZoom >= minZoom) && (valueZoom < maxZoom))
                        {
                            $panzoom.panzoom('zoom', parseFloat(valueZoom),
                                             {
                                                 animate: false,
                                                 focal: e
                                             });
                        }

                    });
                }
            }

        })();
    }

    function exitZoom(hotspot)
    {
        isZooming = false;

        zoomIn.removeClass("zoomout");
        zoomIn.addClass("zoomin");

        zoomImage.attr("src", "#");
        zoomImage.css("display", "none");

        imageContainer.css("display", "block");
        hotspotContainer.css("visibility", "visible");
        hotspotCanvas.css("visibility", "visible");
        playPauseControl.css("visibility", "visible");
        prevControl.css("visibility", "visible");
        nextControl.css("visibility", "visible");

        if (hotspot)
        {
            zoomIn.css("visibility", "visible");
        }

        if (zoomedRotation)
        {
            hotspotContainer.attr("style", "");
        }

        if (isMobileBrowser() && isTopWindow)
        {
            hotspotContainer.css({"top": "50%"});
            hotspotContainer.css({"transform": "translateY(-50%)"});
        }

        zoomPanel.css("display", "none");
        
        //
        
        var cs = getComputedStyle(hotspotCanvas[0]);
        var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
        var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

        hotspotContainer.css({"width": widthCanvasTop});
        hotspotContainer.css({"height": heightCanvasTop});

        //reloadHotspotPositions();
        
        //

        hotspotContainer.css("cursor", "-webkit-"+cursorHoverName);
        hotspotContainer.css("cursor", cursorHoverName);

        isFirstDrag = true;
    }

    var displayPreviousFrame = function()
    {
        removeAllTooltips();

        newFrame = true;

        if (isBouncing && isSingleRotation && (bounceRotationCount === 2) && !isBouncingFinished)
        {
            //console.log("finished");
            isBouncingFinished = true;
            //clearInterval(animation);

            doPause();

            return;
        }

        currentImage++;

        if (currentImage >= (imagesCount/numberOfRows))
        {
            if (isBouncing && !isPlaying)
            {
                currentImage = (imagesCount/numberOfRows)-1;
            }
            else
            {
                currentImage = 0;
            }
        }

        if ((currentImage == ((imagesCount/numberOfRows)-1)) && isSingleRotation && !isBouncing && !lastImageChanged)
        {
            playPauseControl.attr("class", "play");
            isPlaying = false;

            clearInterval(animation);
        }

        if ((currentImage == firstImageIndex) && isSingleRotation && !isBouncing && lastImageChanged)
        {
            playPauseControl.attr("class", "play");
            isPlaying = false;

            clearInterval(animation);
        }

        if ((currentImage == ((imagesCount/numberOfRows)-1)) && isBouncing && !isBouncingFinished && !lastImageChanged)
        {
            if (isSingleRotation)
            {
                bounceRotationCount++;
            }

            if (isPlaying)
            {
                clearInterval(animation);
                animation = setInterval(displayNextFrame, playSpeed);
            }
        }

        if ((currentImage == firstImageIndex) && isBouncing && !isBouncingFinished && lastImageChanged)
        {
            if (isSingleRotation)
            {
                bounceRotationCount++;
            }
            if (isPlaying)
            {
                clearInterval(animation);
                animation = setInterval(displayNextFrame, playSpeed);
            }
        }

        if (!isBlendingEnabled)
        {
            var img = multiRowArray[currentRow][currentImage];

            if (!isTopWindow)
            {
                canvasContext.clearRect(0, 0, normalWidth, normalHeight);
                canvasContext.drawImage(img, 0, 0, normalWidth, normalHeight);
            }
            else
            {
                canvasContext.clearRect(0, 0, largeWidth, largeHeight);
                canvasContext.drawImage(img, 0, 0, largeWidth, largeHeight);
            }

            //

            if (zoomedRotation && isZooming)
            {
                var imgLargeSrc = img.src.replace("img/", "imglarge/");

                var origImage = new Image();
                origImage.onload = function()
                {
                    zoomImage.attr("src", this.src);
                };

                origImage.src = imgLargeSrc;
            }

            //
            
            if (isMobileBrowser() && isTopWindow && zoomedRotation && isZooming)
            {
                hotspotContainer.css("top", "");
                        
                hotspotContainer.css("width", "auto");
                hotspotContainer.css("height", "100%");
                        
                reloadHotspotPositions(true);
            }
            else
            {
                displayHotspot(false);
            }
        }
        else
        {
            fade1();
        }

        previousImage = currentImage;
    };

    var displayNextFrame = function()
    {
        removeAllTooltips();

        newFrame = true;

        if (isBouncing && isSingleRotation && (bounceRotationCount === 2) && !isBouncingFinished)
        {
            //console.log("finished");
            isBouncingFinished = true;
            //clearInterval(animation);

            doPause();

            return;
        }

        currentImage--;

        if (currentImage < 0)
        {
            if (isBouncing && !isPlaying)
            {
                currentImage = 0;
            }
            else
            {
                currentImage = (imagesCount/numberOfRows)-1;
            }
        }

        if ((currentImage == 0) && isSingleRotation && !isBouncing && !lastImageChanged)
        {
            playPauseControl.attr("class", "play");
            isPlaying = false;
            clearInterval(animation);
        }

        if ((currentImage == firstImageIndex) && isSingleRotation && !isBouncing && lastImageChanged)
        {
            playPauseControl.attr("class", "play");
            isPlaying = false;
            clearInterval(animation);
        }

        if ((currentImage == 0) && isBouncing && !isBouncingFinished && !lastImageChanged)
        {
            if (isSingleRotation)
            {
                bounceRotationCount++;
            }
            if (isPlaying)
            {
                clearInterval(animation);
                animation = setInterval(displayPreviousFrame, playSpeed);
            }
        }

        if ((currentImage == firstImageIndex) && isBouncing && !isBouncingFinished && lastImageChanged)
        {
            if (isSingleRotation)
            {
                bounceRotationCount++;
            }
            if (isPlaying)
            {
                clearInterval(animation);
                animation = setInterval(displayPreviousFrame, playSpeed);
            }
        }

        if (!isBlendingEnabled)
        {
            var img = multiRowArray[currentRow][currentImage];

            if (!isTopWindow)
            {
                canvasContext.clearRect(0, 0, normalWidth, normalHeight);
                canvasContext.drawImage(img, 0, 0, normalWidth, normalHeight);
            }
            else
            {
                canvasContext.clearRect(0, 0, largeWidth, largeHeight);
                canvasContext.drawImage(img, 0, 0, largeWidth, largeHeight);
            }

            //

            if (zoomedRotation && isZooming)
            {
                var imgLargeSrc = img.src.replace("img/", "imglarge/");

                var origImage = new Image();
                origImage.onload = function()
                {
                    zoomImage.attr("src", this.src);
                };

                origImage.src = imgLargeSrc;
            }

            //

            if (isMobileBrowser() && isTopWindow && zoomedRotation && isZooming)
            {
                hotspotContainer.css("top", "");
                        
                hotspotContainer.css("width", "auto");
                hotspotContainer.css("height", "100%");
                        
                reloadHotspotPositions(true);
            }
            else
            {
                displayHotspot(false);
            }
        }
        else
        {
            fade1();
        }

        previousImage = currentImage;
    };

    var displayRowFrame = function()
    {
        removeAllTooltips();

        newFrame = true;

        if (!isBlendingEnabled)
        {

            var img = multiRowArray[currentRow][currentImage];

            if (!isTopWindow)
            {
                canvasContext.clearRect(0, 0, normalWidth, normalHeight);
                canvasContext.drawImage(img, 0, 0, normalWidth, normalHeight);
            }
            else
            {
                canvasContext.clearRect(0, 0, largeWidth, largeHeight);
                canvasContext.drawImage(img, 0, 0, largeWidth, largeHeight);
            }

            if (isMobileBrowser() && isTopWindow && zoomedRotation && isZooming)
            {
                hotspotContainer.css("top", "");
                        
                hotspotContainer.css("width", "auto");
                hotspotContainer.css("height", "100%");
                        
                reloadHotspotPositions(true);
            }
            else
            {
                displayHotspot(false);
            }
        }
        else
        {
            fade1();
        }
    };

    function fade1()
    {
        window.requestAnimationFrame(fade1);

        now = Date.now();
        delta = now - then;

        if (delta > interval)
        {
            then = now - (delta % interval);

            //

            opacity -= 0.05;

            if (opacity <= 0)
            {
                opacity = 0;
            }

            var img = multiRowArray[currentRow][currentImage];

            canvasContext.globalAlpha -= 0.05;

            if (canvasContext.globalAlpha <= 0)
            {
                canvasContext.globalAlpha = 0;
            }

            if (!isPlaying && !isClicked)
            {
                opacity = 1;
                canvasContext.globalAlpha = 1;
            }

            if (!isTopWindow)
            {
                if (!isPlaying && !isClicked)
                {
                    canvasContext.clearRect(0, 0, normalWidth, normalHeight);
                }

                canvasContext.drawImage(img, 0, 0, normalWidth, normalHeight);
            }
            else
            {
                if (!isPlaying && !isClicked)
                {
                    canvasContext.clearRect(0, 0, largeWidth, largeHeight);
                }

                canvasContext.drawImage(img, 0, 0, largeWidth, largeHeight);
            }

            displayHotspot(false);
        }
    }

    function roundRect(ctx, x, y, width, height, radius, fill, stroke)
    {
        if (typeof stroke == "undefined" ) {
            stroke = true;
        }
        if (typeof radius === "undefined") {
            radius = 5;
        }
        ctx.beginPath();
        ctx.moveTo(x + radius, y);
        ctx.lineTo(x + width - radius, y);
        ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
        ctx.lineTo(x + width, y + height - radius);
        ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
        ctx.lineTo(x + radius, y + height);
        ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
        ctx.lineTo(x, y + radius);
        ctx.quadraticCurveTo(x, y, x + radius, y);
        ctx.closePath();
        if (stroke) {
            ctx.stroke();
        }
        if (fill) {
            ctx.fill();
        }
    }

    function hexToRgb(hex) {
  var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
  return result ? {
    r: parseInt(result[1], 16),
    g: parseInt(result[2], 16),
    b: parseInt(result[3], 16)
  } : null;
}

    function hideHotspotTitleText()
    {
        tooltipDiv.css("display", "none");
    }

    function displayHotspotTitleText(e, titleText, hotspotFont, hotspotFontSize)
    {
        tooltipDiv.css("font-family", '"' + hotspotFont + '"');
        tooltipDiv.css("font-size", hotspotFontSize);
        tooltipDiv.html(titleText);
        tooltipDiv.css("display", "block");

        var tooltipDivWidth = tooltipDiv.outerWidth();
        var tooltipDivHeight = tooltipDiv.outerHeight();
        var cursorOffsetX = -(tooltipDivWidth/2);
        var cursorOffsetY = -(tooltipDivHeight+(tooltipDivHeight/2));
        var cursorOffsetYBottom = -(tooltipDivHeight/2);
        var canvasOffset = hotspotCanvas.offset();
        var l = canvasOffset.left - cursorOffsetX;
        var t = canvasOffset.top - cursorOffsetY;
        var h = hotspotCanvas.outerHeight();
        var w = hotspotCanvas.outerWidth();

        var maxx = l + w - tooltipDivWidth;
        var maxy = t + h - tooltipDivHeight;

        if (e.clientY <= maxy && e.clientY >= t)
        {
            tooltipDiv.css(
            {
                top: (e.clientY + cursorOffsetY) + 'px'
            });

            tooltipDiv.removeClass("tooltip-arrow-top");
            tooltipDiv.addClass("tooltip-arrow-bottom");
        }
        else
        {
            tooltipDiv.css(
            {
                top: (e.clientY - (cursorOffsetYBottom)) + 'px'
            });

            tooltipDiv.removeClass("tooltip-arrow-bottom");
            tooltipDiv.addClass("tooltip-arrow-top");
        }

        if (e.clientX <= maxx && e.clientX >= l)
        {
            tooltipDiv.css(
            {
                left: (e.clientX + cursorOffsetX) + 'px'
            });
        }
    }

    function displayHotspot(zoomedRotation)
    {
        if (rowsCols !== "undefined")
        {
            canvas = document.getElementById("hotspotcanvas");

            /*
            if (!isTopWindow)
            {
                canvas.width = normalWidth;
                canvas.height = normalHeight;
            }
            else
            {
                canvas.width = largeWidth;
                canvas.height = largeHeight;
            }
            */

            var context = canvas.getContext("2d");
            //context.clearRect(0, 0, canvas.width, canvas.height);

            var img = multiRowArray[currentRow][currentImage];

            if (!isTopWindow)
            {
                canvasContext.drawImage(img, 0, 0, normalWidth, normalHeight);
            }
            else
            {
                canvasContext.drawImage(img, 0, 0, largeWidth, largeHeight);
            }

document.getElementById("hotspotContainer").innerHTML = "";

            for (var row=0; row<rowsCols.length; row++)
            {
                if (row === currentRow)
                {
                    var hotspots = rowsCols[row];

                    for (var h=0; h<hotspots.length; h++)
                    {
                        var hotspot = hotspots[h];

                        var hotspotId = hotspot.hotspotId;
                        var hotspotRow = hotspot.hotspotRow;
                        var hotspotColumn = hotspot.hotspotColumn;
                        var hotspotPositionX = hotspot.hotspotPositionX;
                        var hotspotPositionY = hotspot.hotspotPositionY;
                        var hotspotWidth = hotspot.hotspotWidthSize;
                        var hotspotHeight = hotspot.hotspotHeightSize;
                        var hotspotType = hotspot.hotspotType;
                        var hotspotImage = "hotspots/" + hotspot.hotspotImage;
                        var hotspotTitle = hotspot.hotspotTitle;
                        var hotspotAction = hotspot.hotspotAction;
                        var hotspotActionWindow = hotspot.hotspotActionWindow;
                        var hotspotDescription = hotspot.hotspotDescription;
                        var hotspotFont = hotspot.hotspotFont;
                        var hotspotFontSize = hotspot.hotspotFontSize;

                        if (hotspotFont === "Default")
                        {
                            hotspotFont = "Arial";
                        }

                        if (hotspotColumn === currentImage)
                        {
                            var img = new Image();
                            img.src = hotspotImage;

                            var cs = getComputedStyle(hotspotCanvas[0]);
                            
                            if (zoomedRotation)
                            {
                                cs = getComputedStyle($("body")[0]);
                            }
                            
                            var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
                            var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);

                            if (!isTopWindow)
                            {
                                //hotspotPositionX = hotspotPositionX * normalWidth;
                                //hotspotPositionY = hotspotPositionY * normalHeight;

                                hotspotPositionX = hotspotPositionX * widthCanvasTop;
                                hotspotPositionY = hotspotPositionY * heightCanvasTop;
                            }
                            else
                            {
                                hotspotPositionX = hotspotPositionX * widthCanvasTop;
                                hotspotPositionY = hotspotPositionY * heightCanvasTop;
                            }

var hdiv = document.createElement('div');
//hdiv.title = hotspotTitle;
hdiv.style.position = "absolute";
hdiv.style.left = hotspotPositionX + "px";
hdiv.style.top = hotspotPositionY + "px";
hdiv.style.width = hotspotWidth;
hdiv.style.height = hotspotHeight;
hdiv.style.transform = "translate(-50%, -50%)";
hdiv.style.cursor = "pointer";
hdiv.setAttribute("data-title", hotspotTitle);
hdiv.setAttribute("data-font", hotspotFont);
hdiv.setAttribute("data-fontsize", hotspotFontSize);

//

hdiv.setAttribute("data-hotspotaction", hotspotAction);
hdiv.setAttribute("data-hotspotactionwindow", hotspotActionWindow);
hdiv.setAttribute("data-hotspottitle", hotspotTitle);
hdiv.setAttribute("data-hotspotdescription", hotspotDescription);
hdiv.setAttribute("data-hotspottype", hotspotType);

//

var himage = new Image();
himage.src = hotspotImage;
himage.style.position = "relative";
himage.style.width = "100%";
himage.style.height = "100%";

himage.classList.add("animate__animated");
himage.classList.add("animate__infinite");

var animProcessed = hotspot.hotspotAnimation[0].toLowerCase() + hotspot.hotspotAnimation.substring(1);
var animationCssName = "animate__" + animProcessed;

himage.classList.add(animationCssName);

hdiv.appendChild(himage);
document.getElementById("hotspotContainer").appendChild(hdiv);

$(hdiv).mouseover(function ()
{
    cancelAllAnimationFrames();
});

$(hdiv).on("mouseup touchend", function (e)
{
    switch (parseInt(this.getAttribute("data-hotspottype")))
                            {
                            case 0:
                                break;

                            case 1:

                                hideHotspotTitleText();

                                if (this.getAttribute("data-hotspotaction").indexOf("youtube.com") !== -1)
                                {
                                    var dialogTitle = this.getAttribute("data-hotspottitle");

                                    $("#myDialog").empty();

                                    $("<div></div>").attr({
                                    'id':'player',
                                    'data-plyr-embed-id':this.getAttribute("data-hotspotaction"),
                                    'data-plyr-provider':'youtube'
                                    }).appendTo("#myDialog");

                                    const player = new Plyr('#player');

                                    $("#myDialog").dialog("option", "title", dialogTitle).dialog("open");
                                }
                                else if (this.getAttribute("data-hotspotaction").indexOf("vimeo.com") !== -1)
                                {
                                    var dialogTitle = this.getAttribute("data-hotspottitle");

                                    $("#myDialog").empty();

                                    $("<div></div>").attr({
                                    'id':'player',
                                    'data-plyr-embed-id':this.getAttribute("data-hotspotaction"),
                                    'data-plyr-provider':'vimeo'
                                    }).appendTo("#myDialog");

                                    const player = new Plyr('#player');

                                    $("#myDialog").dialog("option", "title", dialogTitle).dialog("open");
                                }
                                else
                                {
                                    if (this.getAttribute("data-hotspotactionwindow").indexOf("_self") !== -1)
                                    {
                                        window.location.href = this.getAttribute("data-hotspotaction");
                                    }
                                    else
                                    {
                                        var wndw = window.open(this.getAttribute("data-hotspotaction"), "_blank");

                                        if (wndw)
                                        {
                                            wndw.focus();
                                        }
                                        else
                                        {
                                            alert("Please allow popups");
                                        }
                                    }
                                }

                                break;

                            case 2:
                                eval(this.getAttribute("data-hotspotaction")[0]);

                                break;

                            case 3:

                                hideHotspotTitleText();

                                var hotspotImagePath = "hotspotimg/" + this.getAttribute("data-hotspotaction");

                                if (this.getAttribute("data-hotspotaction").endsWith(".mp3"))
                                {
                                    var dialogTitle = this.getAttribute("data-hotspottitle") + " - " + this.getAttribute("data-hotspotdescription");

                                    $("#myDialog").empty();

                                    $("<audio></audio>").attr({
                                    'id':'player',
                                    'src':hotspotImagePath,
                                    'controls':'controls'
                                    }).appendTo("#myDialog");

                                    const player = new Plyr('#player');

                                    $(".plyr").attr({
                                    'style':'position: absolute; width: 100%; height: auto; top: 50%; left: 50%; transform: translate(-50%, -50%);'
                                    });

                                    $("#myDialog").dialog("option", "title", dialogTitle).dialog("open");
                                }
                                if (this.getAttribute("data-hotspotaction").endsWith(".mp4"))
                                {
                                    var dialogTitle = this.getAttribute("data-hotspottitle") + " - " + this.getAttribute("data-hotspotdescription");

                                    $("#myDialog").empty();

                                    $("<video></video>").attr({
                                    'id':'player',
                                    'src':hotspotImagePath,
                                    'playsinline':'playsinline',
                                    'controls':'controls'
                                    }).appendTo("#myDialog");

                                    const player = new Plyr('#player');

                                    $("#myDialog").dialog("option", "title", dialogTitle).dialog("open");
                                }
                                else
                                {
                                    zoomImage.css("display", "block");
                                    zoomImage.css("visibility", "hidden");
                                    zoomPanel.css("display", "block");
                                    imageContainer.css("display", "none");

                                    hotspotCanvas.css("visibility", "hidden");

                                    hotspotContainer.css("visibility", "hidden");
                                    playPauseControl.css("visibility", "hidden");
                                    prevControl.css("visibility", "hidden");
                                    nextControl.css("visibility", "hidden");

                                    //

                                    zoomIn.css("visibility", "hidden");
                                    
                                    var img = new Image();
                                    img.setAttribute("data-hotspotaction", this.getAttribute("data-hotspotaction"));
                                    img.setAttribute("data-hotspotactionwindow", this.getAttribute("data-hotspotactionwindow"));
                                    img.setAttribute("data-hotspottitle", this.getAttribute("data-hotspottitle"));
                                    img.setAttribute("data-hotspotdescription", this.getAttribute("data-hotspotdescription"));
                                    img.setAttribute("data-hotspottype", this.getAttribute("data-hotspottype"));

                                    img.onload = function()
                                    {
                                        zoomImage.css("visibility", "visible");
                                    
                                        hotspotsImageWidth = this.width;
                                        hotspotsImageHeight = this.height;

                                        doZoom(true, this.getAttribute("data-hotspotaction"), this.getAttribute("data-hotspotdescription"), false, null);

                                        if (!mouseZoom)
                                        {
                                            $('.zoomdivclose').remove();

                                            zoomDivClose = document.createElement('div');
                                            zoomDivClose.id = 'zoomdivclose';
                                            zoomDivClose.classList.add("zoomdivclose");

                                            var hotspotImageZoomButton = $(zoomDivClose);
                                            hotspotImageZoomButton.on("mouseup touchend", function (e)
                                            {
                                                e.stopPropagation();
                                                exitZoom(true);
                                                topContainerElement.removeChild(zoomDivClose);
                                                exitFromImageHotspot = true;
                                                
                                                currentZoomStep = 0;
                                                
                                                
                                                if (isInFullscreen() || isTopWindow)
                                                {
                                                    setTimeout(function()
                                                    {
                                                        var cs = getComputedStyle(hotspotCanvas[0]);
                                                        var widthCanvasTop = parseInt(cs.getPropertyValue('width'), 10);
                                                        var heightCanvasTop = parseInt(cs.getPropertyValue('height'), 10);
                                                    
                                                        hotspotContainer.css({"width": widthCanvasTop});
                                                        hotspotContainer.css({"height": heightCanvasTop});
                                                    
                                                        reloadHotspotPositions(false);
                                                        
                                                        //
                                                        console.log("aaa");
                                                        hotspotContainer.css({"top": "50%"});
                                                        hotspotContainer.css({"transform": "translateY(-50%)"});
                                                    }, 500);
                                                }
                                                
                                            });

                                            zoomDivCloseDescription = document.createElement('p');
                                            zoomDivCloseDescription.id = 'zoomdivclose-desc';
                                            zoomDivCloseDescription.innerHTML = this.getAttribute("data-hotspotdescription");

                                            zoomDivCloseContainer = document.createElement('div');
                                            zoomDivCloseContainer.id = 'zoomdivclose-container';

                                            zoomDivCloseContainer.appendChild(zoomDivCloseDescription);
                                            zoomDivClose.appendChild(zoomDivCloseContainer);
                                            topContainerElement.appendChild(zoomDivClose);
                                        }
                                    };

                                    img.src = hotspotImagePath;
                                }

                                break;

                            default:
                                break;
                            }

});
                        }
                    }
                }
            }
        }
    }




function getFontData(hId)
{

for (var row=0; row<rowsCols.length; row++)
            {
                    var hotspots = rowsCols[row];

                    for (var h=0; h<hotspots.length; h++)
                    {
                        var hotspot = hotspots[h];

                        var hotspotId = hotspot.hotspotId;
                        var hotspotRow = hotspot.hotspotRow;
                        var hotspotColumn = hotspot.hotspotColumn;
                        var hotspotPositionX = hotspot.hotspotPositionX;
                        var hotspotPositionY = hotspot.hotspotPositionY;
                        var hotspotWidth = hotspot.hotspotWidthSize;
                        var hotspotHeight = hotspot.hotspotHeightSize;
                        var hotspotType = hotspot.hotspotType;
                        var hotspotImage = "hotspots/" + hotspot.hotspotImage;
                        var hotspotTitle = hotspot.hotspotTitle;
                        var hotspotFont = hotspot.hotspotFont;
                        var hotspotFontSize = hotspot.hotspotFontSize;

if (hId === hotspotId)
{
var fontData = {hotspotFont:hotspotFont, hotspotFontSize:hotspotFontSize};
return fontData;
}
                    }
            }

}

function cancelAllAnimationFrames() {
    var id = window.requestAnimationFrame(function () { });
    while (id--) {
        window.cancelAnimationFrame(id);
    }
}

    function step()
    {
        if (isInertiaEnabled)
        {
            requestAnimationFrame(step);

            if (isClicked)
            {
                previous.x = position.x;
                previous.y = position.y;

                position.x = mouse.x;
                position.y = mouse.y;

                velocity.x = ( position.x - previous.x );
                velocity.y = ( position.y - previous.y );
            }
            else
            {
                position.x += velocity.x;
                position.y += velocity.y;

                velocity.x *= friction;
                velocity.y *= friction;
            }

            if (Math.abs(currentPositionX - position.x) >= sensitivity)
            {
                if (currentPositionX - position.x >= sensitivity)
                {
                    if (isPointerDragNormal)
                    {
                        if (!overHotspot)
                        {
                            displayPreviousFrame();
                        }
                    }
                    else
                    {
                        if (!overHotspot)
                        {
                            displayNextFrame();
                        }
                    }
                }
                else
                {
                    if (isPointerDragNormal)
                    {
                        if (!overHotspot)
                        {
                            displayNextFrame();
                        }
                    }
                    else
                    {
                        if (!overHotspot)
                        {
                            displayPreviousFrame();
                        }
                    }
                }

                currentPositionX = position.x;
            }
        }
    }

    function removeAllTooltips()
    {
        $(".ui-tooltip").remove();
    }

    var doPlay = function()
    {
        playPauseControl.attr("class", "pause");
        isPlaying = true;

        if (isRotationDirectionNormal)
        {
            animation = setInterval(displayNextFrame, playSpeed);
        }
        else
        {
            animation = setInterval(displayPreviousFrame, playSpeed);
        }
    }

    var doPause = function()
    {
        playPauseControl.attr("class", "play");
        isPlaying = false;

        if (isBouncing)
        {
            bounceRotationCount = 0;
            isBouncingFinished = false;
        }

        clearInterval(animation);
    }

    if (isPlaying)
    {
        clearInterval(animation);
        doPlay();
    }

    step();
}
